from math import pi

#
# define shapes and transformation
#
# Limited subset of the standard header for geometric shapes (originally by Neil Gershenfeld),
# edited and culled by Matt Keeter.
#
#
# circle(x0, y0, r)
# rectangle(x0, x1, y0, y1)
# triangle(x0, y0, x1, y1, x2, y2) (points in clockwise order)
# add(part1, part2)
# subtract(part1, part2)
# intersect(part1, part2)
# move(part,dx,dy)
# rotate(part, angle)
# rotate_90(part)
# rotate_180(part)
# rotate_270(part)
# reflect_x(part,x0)
# reflect_y(part,y0)
# reflect_xy(part)

True = "1"
False = "0"

def circle(x0, y0, r):
   return "(((X-({0}))*(X-({0})) + (Y-({1}))*(Y-({1}))) <= ({2}*{2}))".format(x0, y0, r)

def rectangle(x0, x1, y0, y1):
   return "((X >= ({0})) & (X <= ({1})) & (Y >= ({2})) & (Y <= ({3})))".format(x0, x1, y0, y1)

def right_triangle(x0, y0, l):
   return "((X > {0}) & (X < {0} + {2} - (Y-{1})) & {1})".format(x0, y0, l)
   
def triangle(x0, y0, x1, y1, x2, y2): # points in clockwise order
   return ("((((({y1})-({y0}))*(X-({x0}))-(({x1})-({x0}))*(Y-({y0}))) >= 0) & " + \
           "(((({y2})-({y1}))*(X-({x1}))-(({x2})-({x1}))*(Y-({y1}))) >= 0) & " + \
           "(((({y0})-({y2}))*(X-({x2}))-(({x0})-({x2}))*(Y-({y2}))) >= 0))").format(x0 = x0,
                                                                                     y0 = y0,
                                                                                     x1 = x1,
                                                                                     y1 = y1,
                                                                                     x2 = x2,
                                                                                     y2 = y2)

def add(part1, part2):
   if type(part1) is not str or type(part2) is not str:
      raise TypeError("Invalid input types for 'add'")
   return "{0} | {1}".format(part1, part2)

def subtract(part1, part2):
   if type(part1) is not str or type(part2) is not str:
      raise TypeError("Invalid input types for 'subtract'")
   return "({0}) & ~({1})".format(part1, part2)

def intersect(part1, part2):
   if type(part1) is not str or type(part2) is not str:
      raise TypeError("Invalid input types for 'intersect'")
   return "({0}) & ({1})".format(part1, part2)

def move(part,dx,dy):
   part = part.replace('X','(X-('+str(dx)+'))')
   part = part.replace('Y','(Y-('+str(dy)+'))')
   return part   

def rotate(part, angle):
   angle = str(angle*pi/180)
   part = part.replace('X','(cos({0})*X+sin({0})*y)'.format(angle))
   part = part.replace('Y','(-sin({0})*X+cos({0})*y)'.format(angle))
   part = part.replace('y','Y')
   return part

def rotate_90(part):
   part = reflect_y(part,0)
   part = reflect_xy(part)
   return part

def rotate_180(part):
   part = rotate_90(part)
   part = rotate_90(part)
   return part

def rotate_270(part):
   part = rotate_90(part)
   part = rotate_90(part)
   part = rotate_90(part)
   return part

def reflect_x(part,x0):
   return part.replace('X','({!s}-X)'.format(x0))

def reflect_y(part,y0):
   return part.replace('Y','({!s}-Y)'.format(y0))

def reflect_xy(part):
   part = part.replace('X','temp')
   part = part.replace('Y','X')
   part = part.replace('temp','Y')
   return part